@@ -292,6 +292,10 @@ getSection <- function(sectionHeader, outfiletext, headers="standard", omit=NULL
         "FINAL CLASS COUNTS AND PROPORTIONS FOR EACH LATENT CLASS VARIABLE",
         "CLASSIFICATION OF INDIVIDUALS BASED ON THEIR MOST LIKELY LATENT CLASS MEMBERSHIP",
         "Average Latent Class Probabilities for Most Likely Latent Class Membership \\(Row\\)",
+        "Classification Probabilities for the Most Likely Latent Class Membership \\(Row\\)",
+        "Classification Probabilities for the Most Likely Latent Class Membership \\(Column\\)",
+        "Logits for the Classification Probabilities for the Most Likely Latent Class Membership \\(Row\\)",
+        "Logits for the Classification Probabilities for the Most Likely Latent Class Membership \\(Column\\)",
         "MODEL RESULTS", "LOGISTIC REGRESSION ODDS RATIO RESULTS", "RESULTS IN PROBABILITY SCALE",
         "IRT PARAMETERIZATION IN TWO-PARAMETER LOGISTIC METRIC",
         "IRT PARAMETERIZATION IN TWO-PARAMETER PROBIT METRIC",
@@ -308,7 +312,10 @@ getSection <- function(sectionHeader, outfiletext, headers="standard", omit=NULL
         "CONFIDENCE INTERVALS IN PROBABILITY SCALE",
         "CONFIDENCE INTERVALS OF TOTAL, TOTAL INDIRECT, SPECIFIC INDIRECT, AND DIRECT EFFECTS",
         "CONFIDENCE INTERVALS OF STANDARDIZED TOTAL, TOTAL INDIRECT, SPECIFIC INDIRECT,", #omitted "AND DIRECT EFFECTS"
-        "EQUALITY TESTS OF MEANS",
+        "EQUALITY TESTS OF MEANS ACROSS CLASSES USING POSTERIOR PROBABILITY-BASED",
+        "EQUALITY TESTS OF MEANS ACROSS CLASSES USING THE BCH PROCEDURE",
+        "EQUALITY TESTS OF MEANS ACROSS CLASSES USING THE 3-STEP PROCEDURE",
+        "EQUALITY TESTS OF MEANS/PROBABILITIES ACROSS CLASSES",
         "THE FOLLOWING DATA SET\\(S\\) DID NOT RESULT IN A COMPLETED REPLICATION:",
         "RESIDUAL OUTPUT", "MODEL MODIFICATION INDICES", "MODEL COMMAND WITH FINAL ESTIMATES USED AS STARTING VALUES",
         "Available post-processing tools:",
@@ -319,7 +326,22 @@ getSection <- function(sectionHeader, outfiletext, headers="standard", omit=NULL
   
   if (!is.null(omit)) headers <- headers[which(!headers %in% omit)] #drop omit
   
-  beginSection <- grep(sectionHeader, outfiletext, perl=TRUE)[1]
+  #allow for syntax to include :: to specify a header that spans 2 rows. Example:
+  #FINAL CLASS COUNTS AND PROPORTIONS FOR THE LATENT CLASSES
+  #BASED ON THE ESTIMATED MODEL
+
+  #note that this will identify a unique match for the target sectionHeader, but the search for
+  #subsequent headers just uses the one-row list above. For now, this works in all cases I know of.
+  if (grepl("::", sectionHeader, fixed=TRUE)) {
+    firstLine <- sub("^(.*)::.*$", "\\1", sectionHeader, perl=TRUE)
+    nextLine <- sub("^.*::(.*)$", "\\1", sectionHeader, perl=TRUE)
+    candidates <- grep(firstLine, outfiletext, perl=TRUE) #will return NA if no match
+    bothMatch <- grep(nextLine, outfiletext[candidates+1], perl=TRUE)[1] #return first match among candidates
+    if (!is.na(bothMatch)) { beginSection <- candidates[bothMatch] + 1 #since it's a two-line header, skip the first to match typical case
+    } else { beginSection <- NA } #could not find section with both rows
+  } else {
+    beginSection <- grep(sectionHeader, outfiletext, perl=TRUE)[1]  
+  }
   
   #if section header cannot be found, then bail out
   if (is.na(beginSection)) return(NULL)
@@ -438,6 +460,11 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
           identical (nextLine, c("Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%")))
         varNames <- c("param", "est", "posterior_sd", "pval", "lower_2.5ci", "upper_2.5ci")
 
+      #Bayesian 6-column output for R-SQUARE (Just has "Variable" on the front)
+      if (identical(thisLine, c("Posterior", "One-Tailed", "95%", "C.I.")) &&
+          identical (nextLine, c("Variable", "Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%")))
+        varNames <- c("param", "est", "posterior_sd", "pval", "lower_2.5ci", "upper_2.5ci")
+      
       #Bayesian (ESTIMATOR=BAYES) 7-column output (Mplus v7)
       else if (identical(thisLine, c("Posterior", "One-Tailed", "95%", "C.I.")) &&
           identical (nextLine, c("Estimate", "S.D.", "P-Value", "Lower", "2.5%", "Upper", "2.5%", "Significance")))
@@ -457,7 +484,32 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
       else if (identical(thisLine, c("Two-Tailed")) &&
           identical(nextLine, c("Estimate", "S.E.", "Est./S.E.", "P-Value")))
         varNames <- c("param", "est", "se", "est_se", "pval")
+      
+      #Five-column output for R-Square that applies to most unstandardized and standardized sections in Mplus 5 and later
+      else if ((identical(thisLine, c("Observed", "Two-Tailed")) || identical(thisLine, c("Latent", "Two-Tailed"))) &&
+          identical(nextLine, c("Variable", "Estimate", "S.E.", "Est./S.E.", "P-Value")))
+        varNames <- c("param", "est", "se", "est_se", "pval")
 
+      #6-column R-Square output for model with scale factors
+      else if ((identical(thisLine, c("Observed", "Two-Tailed", "Scale")) || identical(thisLine, c("Latent", "Two-Tailed", "Scale"))) &&
+          identical(nextLine, c("Variable", "Estimate", "S.E.", "Est./S.E.", "P-Value", "Factors")))
+        varNames <- c("param", "est", "se", "est_se", "pval", "scale_f")
+      
+      #6-column R-Square output for model with residual variances
+      else if ((identical(thisLine, c("Observed", "Two-Tailed", "Residual")) || identical(thisLine, c("Latent", "Two-Tailed", "Residual"))) &&
+          identical(nextLine, c("Variable", "Estimate", "S.E.", "Est./S.E.", "P-Value", "Variance")))
+        varNames <- c("param", "est", "se", "est_se", "pval", "resid_var")
+      
+      #2-column R-Square without estimates of uncertainty and p-values
+      else if ((identical(thisLine, c("Observed")) || identical(thisLine, c("Latent"))) &&
+          identical(nextLine, c("Variable", "Estimate")))
+        varNames <- c("param", "est")
+
+      #3-column R-Square output for model with residual variances
+      else if ((identical(thisLine, c("Observed", "Residual")) || identical(thisLine, c("Latent", "Residual"))) &&
+          identical(nextLine, c("Variable", "Estimate", "Variance")))
+        varNames <- c("param", "est", "resid_var")
+      
       #Just estimate available, such as in cases of nonconverged models
       else if (identical(thisLine, c("Estimate")))
         varNames <- c("param", "est")
@@ -517,8 +569,11 @@ detectColumnNames <- function(filename, modelSection, sectionType="model_results
     line <- line + 1
     if (exists("varNames"))
       detectionFinished <- TRUE
-    else if (line > length(modelSection))
-      stop("Unable to determine column names for section ", sectionType, ".\n  ", filename)
+    else if (line > length(modelSection)) {
+      warning("Unable to determine column names for section ", sectionType, ".\n  ", filename)
+      return(NULL)
+    }
+      
 
   }
 
@@ -534,3 +589,9 @@ trimSpace <- function(string) {
       }, USE.NAMES=FALSE)
   return(stringTrim)
 }
+
+#helper function to convert strings formatted in Mplus Fortran-style scientific notation using D to indicate double.
+mplus_as.numeric <- function(vec) {
+  vec <- sub("D", "E", vec, fixed=TRUE)
+  as.numeric(vec)
+}
\ No newline at end of file
